# The MIT License (MIT)
#
# Copyright (c) 2016-2017 Thorsten Simons (sw@snomis.de)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import atexit
import os
import sys
import logging
try:
    import readline
except ImportError:
    readline = False

import aw


def main():
    opts = aw.tools.parseargs()
    if opts.debug:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)

    if readline:
        histfile = os.path.join(os.path.expanduser("~"), ".awftp_history")
        # noinspection PyBroadException
        while True:
            try:
                readline.read_history_file(histfile)
                # default history len is -1 (infinite), which may grow unruly
                readline.set_history_length(1000)
            except Exception as e:
                # if this goes wrong, most likely the history file doesn't exist,
                # so let's try and create it
                try:
                    with open(os.path.join(os.path.expanduser("~"),
                                           ".awftp_history"), 'w') as histhdl:
                        pass
                except Exception as f:
                    print('error: open of history file failed ({})...\n\t'
                          'and creating it failed as well ({})\n\t'
                          'That mean\s we can\'t store command history, sorry.'
                          .format(e, f))
                    break
            else:
                atexit.register(readline.write_history_file, histfile)
                break

    try:
        aw.cmd.Awftpshell(completekey='Tab',
                          target=opts.aw_server,
                          nossl=opts.nossl,
                          api=opts.api).cmdloop()
    except UnicodeEncodeError as e:
        fatal = ('Fatal: UnicodeEncodeError\n'
                 '\tMake sure you have your system language set to UTF-8\n'
                 '\tExample:\n'
                 '\t\t$ export LC_ALL=en_US.UTF-8\n'
                 '\n')
        sys.exit(fatal)

if __name__ == '__main__':
    main()



# The MIT License (MIT)
#
# Copyright (c) 2016-2017 Thorsten Simons (sw@snomis.de)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from sys import __stdout__
from io import IOBase
from os import stat
from click import progressbar

from time import sleep

class R(IOBase):

    def __init__(self):
        super().__init__()
        self.isclosed = True
        self.atend = True

    def open(self, filename):
        size = stat(filename).st_size
        self.got = 0
        self.bar = progressbar(length=size, label='test', file=__stdout__)
        self.hdl = open(filename, 'rb')
        self.isclosed = False
        self.atend = False

    def close(self):
        self.hdl.close()
        self.isclosed = False
        self.bar.render_finish()

    def read(self):
        if self.atend or self.isclosed:
            return
        d = self.hdl.read(4096)
        if not len(d):
            self.atend = True
        self.got += len(d)
        self.bar.update(self.got)
        sleep(.1)
        return(d)
